import 'package:casino_plus/src/apps/parent_app/parent_app.dart';
import 'package:casino_plus/src/config/games.dart';
import 'package:casino_plus/src/config/offline_game_user_seat.dart';
import 'package:casino_plus/src/config/offline_players.dart';
import 'package:casino_plus/src/router/navi_state.dart' as router_state;
import 'package:casino_plus/src/router/navi_window.dart';
import 'package:casino_plus/src/router/page_id.dart';
import 'package:casino_plus/src/router/router.dart';
import 'package:casino_plus/src/state/game_room.dart';
import 'package:casino_plus/src/state/partycake.dart';
import 'package:casino_plus/src/types/game_id.dart';
import 'package:casino_plus/src/types/partycake/seat.dart';
import 'package:casino_plus/src/types/player.dart';
import 'package:casino_plus/src/types/user_type.dart';
import 'package:flutter/material.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import 'package:uuid/uuid.dart';

class GameOfflinePage extends StatelessWidget {
  // ゲームID
  final GameId gameId;
  // ページを閉じるとき
  final void Function() onGamePageClose;
  GameOfflinePage({
    required this.gameId,
    required this.onGamePageClose,
  }) {
    ParentApp.onGamePageClose = onGamePageClose;
  }
  @override
  Widget build(BuildContext context) {
    final _page = _CasinoPlusOfflinePage(
      gameId: gameId,
    );
    final scope = ProviderScope(child: _page);
    return scope;
  }
}

// ProviderScope で囲むために分離
class _CasinoPlusOfflinePage extends HookConsumerWidget {
  final GameId gameId;
  const _CasinoPlusOfflinePage({
    required this.gameId,
  });
  @override
  Widget build(BuildContext context, WidgetRef ref) {
    useEffect(() {
      WidgetsBinding.instance?.addPostFrameCallback((_) {
        debugPrint('Casino+(Offline) did builded.');

        // 適当なプレイヤーを そのゲームの最大人数分 -1 つくる
        final playerCount =
            games.firstWhere((game) => game.id == gameId).maxPlayerCount;

        final offlineSeat = offlineGameUserSeat(gameId);
        final myUserId = const Uuid().v4();
        final offlinePlayer = Player(
          seat: offlineSeat.rawValue,
          userId: myUserId,
          nickname: 'あなた',
          iconUrl: 'assets://icon-guest.png',
          userType: UserType.human,
        );

        final players = List.generate(playerCount, (index) {
          final seat = index += 1;
          final botPlayer = botPlayers.firstWhere((bot) {
            return bot.seat == seat;
          });
          return seat == offlineSeat.rawValue ? offlinePlayer : botPlayer;
        });

        final newGameRoomState = GameRoomState(
          gameId: gameId,
          myUserId: myUserId,
          hostUserId: myUserId,
          players: players,
        );
        ref.read(gameRoomState.notifier).update(newGameRoomState);

        final oldPartycakeState = ref.read(partycakeState);
        final newPartycakeState = PartycakeState(
          record: oldPartycakeState.record,
        );
        ref.read(partycakeState.notifier).update(newPartycakeState);

        ref.read(router).pushPage(PageId.gamePlay);
      });
    }, const []);

    final naviState = ref.watch(router_state.naviState);
    final naviWindow = NaviWindow(
      state: naviState,
      popPage: () {
        ref.read(router).popPage();
      },
    );
    final scaffold = Scaffold(body: naviWindow);
    return scaffold;
  }
}
