import 'package:casino_plus/src/host/partycake/logger.dart';
import 'package:casino_plus/src/host/partycake/partycake_core/runner.dart';
import 'package:casino_plus/src/host/partycake/record_repository.dart';
import 'package:casino_plus/src/senders/partycake/dealer_did_showdown_sender.dart';
import 'package:casino_plus/src/senders/partycake/dealer_did_shuffle_sender.dart';
import 'package:casino_plus/src/senders/partycake/player_did_bet_sender.dart';
import 'package:casino_plus/src/senders/partycake/player_did_put_sender.dart';
import 'package:casino_plus/src/types/playing_card.dart';
import 'package:casino_plus/src/types/partycake/bet_level.dart';
import 'package:casino_plus/src/types/partycake/seat.dart';
import 'package:casino_plus/src/types/partycake/record.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';

final partycakeController = Provider((ref) => _PartycakeController(ref.read));

class _PartycakeController implements PartycakeObserver {
  final Reader read;
  _PartycakeController(this.read);
  betAction(Seat seat, BetLevel level) {
    final repository = RecordRepository();
    final record = repository.loadRecord();
    final logger = Logger();
    final controller = PartycakeRunner(
      observers: [
        logger,
        this,
      ],
    );
    controller.betAction(seat: seat, level: level, forRecord: record);
  }

  putAction(Seat seat, PlayingCard cardId) {
    final repository = RecordRepository();
    final record = repository.loadRecord();
    final logger = Logger();
    final controller = PartycakeRunner(
      observers: [
        logger,
        this,
      ],
    );
    controller.putAction(seat: seat, cardId: cardId, forRecord: record);
  }

  @override
  void dealerDidShowdown(Record record) {
    final repository = RecordRepository();
    repository.saveRecord(record);

    final sender = DealerDidShowdownSender(read);
    sender.sendDealerDidShowdownMessage(record);
  }

  @override
  void dealerDidShuffle(Record record) {
    final repository = RecordRepository();
    repository.saveRecord(record);

    final sender = DealerDidShuffleSender(read);
    sender.sendDealerDidShuffleMessgae(record);
  }

  @override
  void playerDidBet(Record record, Seat seat) {
    final repository = RecordRepository();
    repository.saveRecord(record);

    final sender = PlayerDidBetSender(read);
    sender.sendPlayerDidBetMessage();
  }

  @override
  void playerDidPut(Record record, Seat seat) {
    final repository = RecordRepository();
    repository.saveRecord(record);

    final sender = PlayerDidPutSender(read);
    sender.sendPlayerDidPutMessage(seat);
  }
}
