import 'package:casino_plus/src/host/partycake/partycake_core/bet.dart';
import 'package:casino_plus/src/host/partycake/partycake_core/put.dart';
import 'package:casino_plus/src/host/partycake/partycake_core/showdown.dart';
import 'package:casino_plus/src/host/partycake/partycake_core/shuffle.dart';
import 'package:casino_plus/src/host/partycake/partycake_core/step_validations.dart';
import 'package:casino_plus/src/types/playing_card.dart';
import 'package:casino_plus/src/types/partycake/bet_level.dart';
import 'package:casino_plus/src/types/partycake/record.dart';
import 'package:casino_plus/src/types/partycake/seat.dart';

// TODO: - ここで masked Record 渡す
abstract class PartycakeObserver {
  void playerDidBet(Record record, Seat seat);
  void dealerDidShuffle(Record record);
  void playerDidPut(Record record, Seat seat);
  void dealerDidShowdown(Record record);
}

// TODO: - コントローラーではなくただの公開部分インターフェース。タスクランナー？トリガー?ディスパッチャー?reducer?
class PartycakeRunner {
  final List<PartycakeObserver> observers;
  PartycakeRunner({required this.observers});

  void _notifyPlayerDidBet(Record record, Seat seat) {
    for (var observer in observers) {
      observer.playerDidBet(record, seat);
    }
  }

  void _notifyDealerDidShuffle(Record record) {
    for (var observer in observers) {
      observer.dealerDidShuffle(record);
    }
  }

  void _notifyPlayerDidPut(Record record, Seat seat) {
    for (var observer in observers) {
      observer.playerDidPut(record, seat);
    }
  }

  void _notifyDealerDidShowdown(Record record) {
    for (var observer in observers) {
      observer.dealerDidShowdown(record);
    }
  }

  // 公開メソッド
  betAction({
    required Seat seat,
    required BetLevel level,
    required Record forRecord,
  }) {
    var record = forRecord;
    final _canBet = canBet(level: level, seat: seat, record: record);
    if (_canBet) {
      // Bet
      record = bet(seat, level, record);
      _notifyPlayerDidBet(record, seat);
    } else {
      throw Exception('不正なBetです');
    }

    final _canShuffle = canShuffle(record: record);
    if (_canShuffle) {
      // Shuffle
      record = shuffle(record);
      _notifyDealerDidShuffle(record);
    }
  }

  // 公開メソッド
  putAction({
    required Seat seat,
    required PlayingCard cardId,
    required Record forRecord,
  }) {
    var record = forRecord;
    final _canPut = canPut(cardId: cardId, seat: seat, record: record);
    if (_canPut) {
      // Put
      record = put(seat, cardId, record);
      _notifyPlayerDidPut(record, seat);
    } else {
      throw Exception('不正なPutです');
    }

    final _canShowdown = canShowdown(record: record);
    if (_canShowdown) {
      record = showdown(record);
      _notifyDealerDidShowdown(record);
    }
  }
}
