//
//
//

import 'package:casino_plus/src/config/cards.dart';
import 'package:casino_plus/src/state/partycake.dart';
import 'package:casino_plus/src/state/partycake_ui_state.dart';
import 'package:casino_plus/src/types/number.dart';
import 'package:casino_plus/src/types/partycake/seat.dart';
import 'package:casino_plus/src/types/partycake/side.dart';
import 'package:riverpod/riverpod.dart';

class ShowdownPresenter {
  final Reader _read;
  ShowdownPresenter(this._read);

  showdown() async {
    final gameState = _read(partycakeState);
    final sides = gameState.record!.sides;
    sides.sort((s0, s1) => s0.seat.rawValue.compareTo(s1.seat.rawValue));
    for (var side in sides) {
      await showdownForSide(side);
    }
  }

  showdownForSide(Side side) async {
    // カードをめくる
    await openPutCardForSide(side);

    var rotateInner = false;
    var rotateOuter = false;
    var card = cards.firstWhere((c) => c.id == side.putCardId);
    switch (card.number) {
      case Number.num11:
        rotateInner = true;
        break;
      case Number.num12:
        rotateOuter = true;
        break;
      case Number.num13:
        rotateInner = true;
        rotateOuter = true;
        break;
      default:
        break;
    }

    final oldUiState = _read(partycakeUiState);
    var innerDegree = oldUiState.innerCakeDegree;
    var outerDegree = oldUiState.outerCakeDegree;
    if (rotateInner) {
      // rotate
      outerDegree += 90;
    }
    if (rotateOuter) {
      // rotate
      innerDegree += 90;
    }
    if (rotateInner || rotateOuter) {
      final newUiState = oldUiState.copyWith(
        innerCakeDegree: innerDegree,
        outerCakeDegree: outerDegree,
      );
      _read(partycakeUiState.notifier).update(newUiState);
      await Future.delayed(const Duration(seconds: 1));
    }
  }

  openPutCardForSide(Side side) async {
    final oldUiState = _read(partycakeUiState);
    final oldSideUiState =
        oldUiState.uiSides.firstWhere((uiSide) => uiSide.seat == side.seat);
    final newSideUiState = oldSideUiState.copyWith(
      putCardId: side.putCardId,
      putCardDegree: 0,
    );
    var newUiSides = List.of(oldUiState.uiSides);
    newUiSides.removeWhere((uiSide) => uiSide.seat == side.seat);
    newUiSides.add(newSideUiState);
    final newUiState = oldUiState.copyWith(uiSides: newUiSides);
    _read(partycakeUiState.notifier).update(newUiState);
    await Future.delayed(const Duration(seconds: 1));
  }
}

final showdownPresenter = Provider((ref) => ShowdownPresenter(ref.read));
