//
//
//

import 'package:casino_plus/src/state/game_room.dart';
import 'package:casino_plus/src/state/partycake.dart';
import 'package:casino_plus/src/state/partycake_side_ui_state.dart';
import 'package:casino_plus/src/state/partycake_ui_state.dart';
import 'package:casino_plus/src/types/partycake/game_message.dart';
import 'package:casino_plus/src/types/partycake/seat.dart';
import 'package:flutter/material.dart';
import 'package:riverpod/riverpod.dart';

class DealerDidShuffleReceiver {
  final Reader read;
  DealerDidShuffleReceiver(this.read);

  onReceiveDealerDidShuffle(GameMessage message) async {
    debugPrint('Partycake onReceiveDealerDidShuffle');

    // 内部システムデータ
    final newGameState = PartycakeState(
      record: message.record!,
    );
    read(partycakeState.notifier).update(newGameState);

    // Ui の更新
    final gameState = read(partycakeState);
    final roomState = read(gameRoomState);
    final oldUiState = read(partycakeUiState);
    final player =
        roomState.players.firstWhere((p) => p.userId == roomState.myUserId);
    final side = gameState.record!.sides
        .firstWhere((s) => s.seat.rawValue == player.seat);

    // Handカード
    final cardIds = side.handCardIds;

    // Putされたカードの削除
    List<PartycakeSideUiState> newUiSides = [];
    for (var uiSide in oldUiState.uiSides) {
      final newUiSide = uiSide.copyWith(
        putCardId: null,
        putCardDegree: 90,
      );
      newUiSides.add(newUiSide);
    }
    final newUiState = oldUiState.copyWith(
      dockHandCards: cardIds,
      uiSides: newUiSides,
    );
    read(partycakeUiState.notifier).update(newUiState);
  }
}
