import 'package:casino_plus/src/host/partycake/partycake_core/initial_record.dart';
import 'package:casino_plus/src/state/partycake_side_ui_state.dart';
import 'package:casino_plus/src/types/playing_card.dart';
import 'package:casino_plus/src/types/partycake/bet_level.dart';
import 'package:casino_plus/src/types/partycake/focus_target.dart';
import 'package:freezed_annotation/freezed_annotation.dart';
import 'package:riverpod/riverpod.dart';
part 'partycake_ui_state.freezed.dart';
part 'partycake_ui_state.g.dart';

@freezed
class PartycakeUiState with _$PartycakeUiState {
  const PartycakeUiState._();

  // ignore: invalid_annotation_target
  @JsonSerializable(fieldRename: FieldRename.snake)
  factory PartycakeUiState({
    @FocusTargetConverter() required FocusTarget focusTarget,
    required bool waitingOthers,
    required bool canExit,
    required bool showExitModal,
    required double innerCakeDegree,
    required double outerCakeDegree,
    @PlayingCardConverter() required List<PlayingCard> dockHandCards,
    @BetLevelConverter() required List<BetLevel> dockBetLevels,
    required bool dockIsLocked,
    required PlayingCard? dockSelectedCard,
    required BetLevel? dockSelectedBetLevel,

    // Effect
    required bool spotlightMySide,
    required String? cardEffectImageName,
    required double cardEffectImageSize,
    required String? flatCardImageName,
    required String? flatOuterImageName,
    required String? flatInnerImageName,
    required String? comboName,
    required int comboNameWaveIndex,

    // sides
    required List<PartycakeSideUiState> uiSides,
  }) = _PartycakeUiState;

  factory PartycakeUiState.fromJson(Map<String, dynamic> json) =>
      _$PartycakeUiStateFromJson(json);
}

/// バインド
class _Notifier extends StateNotifier<PartycakeUiState> {
  _Notifier(PartycakeUiState state) : super(state);
  update(PartycakeUiState state) {
    this.state = state;
  }
}

/// 公開
final partycakeUiState =
    StateNotifierProvider<_Notifier, PartycakeUiState>((ref) {
  // 初期化
  final initialState = PartycakeUiState(
    waitingOthers: true,
    canExit: true,
    showExitModal: true,
    innerCakeDegree: 0,
    outerCakeDegree: 0,
    dockHandCards: [],
    dockBetLevels: [],
    dockIsLocked: false,
    dockSelectedCard: null,
    dockSelectedBetLevel: null,

    // Effect
    spotlightMySide: true,
    cardEffectImageName: null,
    cardEffectImageSize: 0,
    flatCardImageName: null,
    flatOuterImageName: null,
    flatInnerImageName: null,
    comboName: null,
    comboNameWaveIndex: 0,
    focusTarget: FocusTarget.all,

    uiSides: initialRecord.sides.map(
      (side) {
        return PartycakeSideUiState(
          seat: side.seat,
          putCardId: null,
          putCardDegree: 0,
        );
      },
    ).toList(),
  );
  return _Notifier(initialState);
});
